///////////
//
// analyticsBS.cpp
//
////////////

/* Code to implement the analytic formulae for certain known derivative payoffs based on the primary Black-Scholes Assumptions */

#include <cmath>
#include <analyticsBS.h>
#include <normals.h>

double BSForward(double Spot,
				 double Strike,
				 double RiskFreeRate,
				 double DividendRate,
				 double TimeToExpiry){
	return (exp(-DividendRate * TimeToExpiry) * Spot - exp(-RiskFreeRate * TimeToExpiry) * Strike);
}

double BSCall(double Spot,
			  double Strike,
			  double RiskFreeRate,
			  double DividendRate,
			  double Volatility,
			  double TimeToExpiry){
	double StandardDeviation = Volatility * sqrt(TimeToExpiry);
	double moneyness = log (Spot/Strike);
	double d1 = (moneyness + (RiskFreeRate - DividendRate) * TimeToExpiry + 0.5 * StandardDeviation * StandardDeviation) / StandardDeviation;
	double d2 = d1 - StandardDeviation;
	return (Spot * exp (-DividendRate * TimeToExpiry) * NormalCDF(d1) - Strike * exp (-RiskFreeRate * TimeToExpiry) * NormalCDF(d2));
}

double BSPut(double Spot,
			 double Strike,
			 double RiskFreeRate,
			 double DividendRate,
			 double Volatility,
			 double TimeToExpiry){
	double StandardDeviation = Volatility * sqrt(TimeToExpiry);
	double moneyness = log (Spot/Strike);
	double d1 = (moneyness + (RiskFreeRate - DividendRate) * TimeToExpiry + 0.5 * StandardDeviation * StandardDeviation) / StandardDeviation;
	double d2 = d1 - StandardDeviation;
	return (Strike * exp (-RiskFreeRate * TimeToExpiry) * (1.0 - NormalCDF(d2)) - Spot * exp (-DividendRate * TimeToExpiry) * (1.0 - NormalCDF(d1)));
	
}

double BSDigitalCall(double Spot,
					 double Strike,
					 double RiskFreeRate,
					 double DividendRate,
					 double Volatility,
					 double TimeToExpiry){
	double StandardDeviation = Volatility * sqrt(TimeToExpiry);
	double moneyness = log (Spot/Strike);
	double d2 = (moneyness + (RiskFreeRate - DividendRate) * TimeToExpiry - 0.5 * StandardDeviation * StandardDeviation) / StandardDeviation;
	return exp(-RiskFreeRate * TimeToExpiry) * NormalCDF(d2);
}

double BSDigitalPut(double Spot,
					double Strike,
					double RiskFreeRate,
					double DividendRate,
					double Volatility,
					double TimeToExpiry){
	double StandardDeviation = Volatility * sqrt(TimeToExpiry);
	double moneyness = log (Spot/Strike);
	double d2 = (moneyness + (RiskFreeRate - DividendRate) * TimeToExpiry - 0.5 * StandardDeviation * StandardDeviation) / StandardDeviation;
	return exp(-RiskFreeRate * TimeToExpiry) * (1.0 - NormalCDF(d2));
}

double BSZeroCouponBond(double RiskFreeRate,
						double TimeToExpiry){
	return exp(-RiskFreeRate * TimeToExpiry);		
}